#include <stdlib.h>
#include "..\lib\UPAC5000.h"
#include "..\lib\XWCAN.h"

unsigned long MsgCount;

/*If the UserCANInt() function is not used, please don't remove it.*/		  

void UserCANInt(int CANPort,char CANInt){  
   if(CANPort ==0x0)
  {
	  if (CANInt==0x01)   /*avoid complier warning*/
	 	{
	 	 MsgCount++;
	 	}
  }
}

void main(void)
{
    int ret,i;
    unsigned char MsgMode,MsgRTR,MsgDataLen,MsgData[8];
    unsigned long MsgID,MsgUpperTimeStamp,MsgLowerTimeStamp,MsgCnt=0;

    InitLib();   

    /********************************************************/
    /*                    Initial all LEDs                  */
    /********************************************************/

    CL1Off();
    CL2Off();
    CL3Off();   

    /*******************************************************/
    /*      initialize and configure the CAN controller    */
    /*******************************************************/		

    ret=XWCANInit(0,1,125000UL,0,0,0x00000000UL,0xffffffffUL);
    /* XCANInit function Parameter descriptions
       0            : for CAN Port selection
       1            : for Receive  interrupt
       125000UL     : for CAN baud
       0            : for BT0 of user defined baud
       0            : for BT1 of user defined baud
       0x00000000UL : for AccCode of CAN message filter
       0xffffffffUL : for AccMask of CAN message filter
    */

    switch (ret){  /*Check if configuration is OK*/
	    case CAN_ResetError:
	 	    Print("Reset Error!\n\r");
		    return;

	    case CAN_SetACRError:
		    Print("\n\rSet ACR Error!");
		    return;

	    case CAN_SetAMRError:
		    Print("\n\rSet AMR Error!");
		    return;

	    case CAN_SetBaudRateError:
		    Print("\n\rSet Baud Rate Error!");
		    return;

	    case CAN_BaudNotSupport:
		    Print("\n\rBaud Rate Not Support!");
		    return;	

	    case CAN_ConfigError:
		    Print("\n\rConfiguration Failure!");
		    return;

	    default:
		    if (ret >0){
			    Print("Default=%d\n\r",ret);
			    return;
		    }
    }

    /********************************************************/
    /*                     Enable Irq                       */
    /********************************************************/

    CAN_CreateBuffer(0,0,1000);   //Create RX Buffer    
    /* CAN_CreateBuffer function parameter descirptions:    	    
	    0            : for CAN Port selection	    
	    0            : Rx buffer	    
	    0            : Rx buffer count	    
	  */

    CAN_InstallIrq(0);     
    /* CAN_InstallIrq function parameter descirptions:	    
	    0            : for CAN Port selection	    
	  */

    Print("Press any key to exit the program.\n\r");

    while(1){	
      /********************************************************/
    	/*          Receive CAN messages from CAN bus           */
    	/********************************************************/
    	ret=GetCANMsg(0,&MsgMode,&MsgID,&MsgRTR,&MsgDataLen,MsgData,&MsgUpperTimeStamp,&MsgLowerTimeStamp);

      if (!ret){
	      MsgCnt++;
		    Print("%lu(%lus-%luus):Mode=%d,ID=%lx,RTR=%d,Len=%d",MsgCnt,MsgUpperTimeStamp
		           ,MsgLowerTimeStamp,MsgMode,MsgID,MsgRTR,MsgDataLen);	

		    if (MsgDataLen && !MsgRTR){
			    Print(",Data=");
			    for (i=0;i<MsgDataLen;i++){
				    Print("%x,",MsgData[i]);
			    }
		    }
		    Print("\n\r");

	    }

	    else{
		    switch(ret){
			    case CAN_DataLengthError:
				    Print("Reception Data Length Error!\n");
				    break;

			    case CAN_DataOverrun:
			    	Print("Software Reception Data Buffer Overrun!\n");
			    	break;

			    case CAN_ReceiveError:
				    Print("Receive data Error!\n");
				    break;					

			    case CAN_ReceiveBufferEmpty: //No message in receive buffer
				    break;
		    }

	    }

	    if (Kbhit()){ /*if press any key, exit the program*/
		    Print("Exit this program!\n");
		    break;
	    }

    }

    Print("%lu CAN message%c %s received.\n\r",MsgCount,(MsgCount<2)?' ':'s',(MsgCount<2)?"is":"are");

     CAN_Restore(0);
    /* CAN_Restore function parameter descirptions:
	    
	    0            : for CAN Port selection
	    
	  */

}

